//- ****************************************************************************
//- 
//- Copyright 2009 Sandia Corporation. Under the terms of Contract 
//- DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains 
//- certain rights in this software.
//-
//- BSD Open Source License.
//- All rights reserved.
//- 
//- Redistribution and use in source and binary forms, with or without 
//- modification, are permitted provided that the following conditions are met:
//-
//-    * Redistributions of source code must retain the above copyright notice, 
//-      this list of conditions and the following disclaimer.
//-    * Redistributions in binary form must reproduce the above copyright 
//-      notice, this list of conditions and the following disclaimer in the 
//-      documentation and/or other materials provided with the distribution.
//-    * Neither the name of Sandia National Laboratories nor the names of its 
//-      contributors may be used to endorse or promote products derived from  
//-      this software without specific prior written permission.
//-
//- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
//- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
//- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//- POSSIBILITY OF SUCH DAMAGE.
//-

#ifndef BaseGlobals_H
#define BaseGlobals_H

// **** _DEFINES_ **************************************************************

#ifndef ABSTRACT
//! Global constant used to make pure virtual functions readable.
#define ABSTRACT                            0
#endif

#ifndef int64
//! Sun defines long as int ... this defines long long as a true long (int64).
typedef long long int64;
#endif

#ifndef uByte
//! Unsigned-byte typedef
typedef unsigned char uByte;
#endif

// **** _GLOBAL CONSTANTS_ *****************************************************

//! \brief Definition of PI
//! 
//! Definition of PI
const double PI          = 3.1415926535897932384626;

//! 2 pi.
const double TWO_PI      = 6.2831853071795864769253;

//! pi/2.
const double PI_OVER_TWO = 1.5707963267948966192313;

//! 
//! Conversion factor from radians to degrees.
const double RAD_TO_DEG  = 180./3.1415926535897932384626;
//const double RAD_TO_DEG  = 57.295779513082320876798;

//! \brief Conversion factor from degrees to radians.
//! 
//! Conversion factor from degrees to radians.
const double DEG_TO_RAD  = 3.1415926535897932384626/180.;
//const double DEG_TO_RAD  = 0.017453292519943295769237;

//! \brief Default constant for 'Not Available'.
//! 
//! Default constant for 'Not Available'.
const double NA_VALUE    = -999999.0;

//! \brief Earth average radius in km.
//!
//! Earth average radius in km.
const double EARTH_RAD   = 6371.0;

//! \brief Earth equatorial radius in km for GRS80 ellipsoid
//!
//! Earth equatorial radius in km for GRS80 ellipsoid
const double EARTH_A     = 6378.137;  // GRS80

//! \brief Ellipsoidal Earth flattening parameter for GRS80 ellipsoid
//! 
//! Ellipsoidal Earth flattening parameter for GRS80 ellipsoid
const double EARTH_F     = 1.0 / 298.257222101;  

//const double EARTH_E   = 2 * EARTH_F - EARTH_F * EARTH_F;
const double EARTH_E     = 0.00669438002290078690;

#endif // BaseGlobals_H
