//- ****************************************************************************
//- 
//- Copyright 2009 Sandia Corporation. Under the terms of Contract 
//- DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains 
//- certain rights in this software.
//-
//- BSD Open Source License.
//- All rights reserved.
//- 
//- Redistribution and use in source and binary forms, with or without 
//- modification, are permitted provided that the following conditions are met:
//-
//-    * Redistributions of source code must retain the above copyright notice, 
//-      this list of conditions and the following disclaimer.
//-    * Redistributions in binary form must reproduce the above copyright 
//-      notice, this list of conditions and the following disclaimer in the 
//-      documentation and/or other materials provided with the distribution.
//-    * Neither the name of Sandia National Laboratories nor the names of its 
//-      contributors may be used to endorse or promote products derived from  
//-      this software without specific prior written permission.
//-
//- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
//- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
//- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//- POSSIBILITY OF SUCH DAMAGE.
//-


//- ****************************************************************************
//-
//- Program:       Parametric Grid Library (PGL)
//- Module:        $RCSfile: CpuTimer.h,v $
//- Creator:       Jim Hipp
//- Creation Date: October 15, 1998
//- Revision:      $Revision: 1.7 $
//- Last Modified: $Date: 2009/08/31 19:19:06 $
//- Last Check-in: $Author: mchang $
//-
//- ****************************************************************************

#ifndef CPU_TIMER_H
#define CPU_TIMER_H

#include <ctime>

using namespace std;

#include "BaseGlobals.h"
#include "SLBMGlobals.h"

namespace slbm {

class SLBM_EXP CpuTimer
{
  public:

    CpuTimer();
    //- default constructor initialies timer

    void              init_timer();
    //- initializes timer

    double            cpu_time();
    double            cpu_time_init();
    //- returns cpu time (seconds) since last init_timer call. The second
    //- function also reinitializes cpuCPUTime.

    double            real_time();
    double            real_time_init();
    //- returns real time (seconds) since last init_timer call. The second
    //- function also reinitializes cpuRealTime.

    double            cpu_nano_time();
    double            cpu_nano_dtime(double tstrt);
    double            cpu_nano_init();
    //- returns the nearest nano second resolution time. This function should always be used as
    //-
    //- CpuTimer cp;
    //- ...
    //- tstrt = cp.cpu_nano_time();
    //- ... // code to be timed
    //- tend  = cp.cpu_nano_time();
    //- telapsed = tend - tstrt;
    //-
    //- or as
    //-
    //- tstrt    = cp.cpu_nano_time();
    //- ... // code to be timed
    //- telapsed = cp.cpu_nano_dtime(tstrt);


  private:

    double     cpuRealTime;
    //- last real time set when init_timer was called or 'this' CpuTimer was
    //- constructed

    double     cpuCPUTime;
    //- last cpu time set when init_timer was called or 'this' CpuTimer was
    //- constructed

    double     cpuNanoTime;
    //- last nano cpu time set when cpu_nano_time() was called.
};

} // end namespace slbm

#endif  // CPU_TIMER_H

