//- ****************************************************************************
//- 
//- Copyright 2009 Sandia Corporation. Under the terms of Contract 
//- DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains 
//- certain rights in this software.
//-
//- BSD Open Source License.
//- All rights reserved.
//- 
//- Redistribution and use in source and binary forms, with or without 
//- modification, are permitted provided that the following conditions are met:
//-
//-    * Redistributions of source code must retain the above copyright notice, 
//-      this list of conditions and the following disclaimer.
//-    * Redistributions in binary form must reproduce the above copyright 
//-      notice, this list of conditions and the following disclaimer in the 
//-      documentation and/or other materials provided with the distribution.
//-    * Neither the name of Sandia National Laboratories nor the names of its 
//-      contributors may be used to endorse or promote products derived from  
//-      this software without specific prior written permission.
//-
//- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
//- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
//- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//- POSSIBILITY OF SUCH DAMAGE.
//-


//- ****************************************************************************
//-
//- Program:       Parametric Grid Library (PGL)
//- Module:        $RCSfile: CpuTimer.cc,v $
//- Creator:       Jim Hipp
//- Creation Date: October 15, 1998
//- Revision:      $Revision: 1.5 $
//- Last Modified: $Date: 2009/08/31 19:19:06 $
//- Last Check-in: $Author: mchang $
//-
//- ****************************************************************************

#include "CpuTimer.h"

// **** _BEGIN SLBM NAMESPACE_ **************************************************

namespace slbm {

CpuTimer::CpuTimer()
{
  // initialize real and cpu times

  init_timer();
}

void CpuTimer::init_timer()
{
  // initialize real and cpu times

  cpuRealTime = time(NULL);
  cpuCPUTime  = clock() / CLOCKS_PER_SEC;
  cpuNanoTime = cpu_nano_time();
}

double CpuTimer::cpu_time()
{
  return ((double) clock() / CLOCKS_PER_SEC - cpuCPUTime);
}

double CpuTimer::cpu_time_init()
{
  // save old time and get new

  double old_time = cpuCPUTime;
  cpuCPUTime = (double) clock() / CLOCKS_PER_SEC;

  // return difference

  return (cpuCPUTime - old_time);
}

double CpuTimer::real_time()
{
  return ((double) time(NULL) - cpuRealTime);
}

double CpuTimer::real_time_init()
{
  // save old time and get new

  double old_time = cpuRealTime;
  cpuRealTime = (double) time(NULL);

  // return difference

  return (cpuRealTime - old_time);
}

/* Linux does not support the gethrtime() function
 * Fortunately, the two nano_time methods are not used anywhere in LocOO,
 * so I have simply commented it out and added a dummy return value
 */
double CpuTimer::cpu_nano_time()
{
  //return (double) gethrtime() * 1.0e-9;
  return(0.0);
}

double CpuTimer::cpu_nano_dtime(double tstrt)
{
  //return ((double) gethrtime() * 1.0e-9 - tstrt);
  return(0.0);
}

double CpuTimer::cpu_nano_init()
{
  // save old time and get new

  double old_time = cpuNanoTime;
  cpuNanoTime = cpu_nano_time();

  // return difference

  return (cpuNanoTime - old_time);
}

} // end namespace slbm
